package net.sf.sido;

import java.math.BigDecimal;

import net.sf.sido.array.Array;

import org.joda.time.DateTime;
import org.joda.time.Duration;
import org.joda.time.Period;

import com.google.common.base.Predicate;

public interface DataTypeBuilder {
	DataTypeBuilder abstractType();

	DataTypeBuilder addAnonymousArrayProperty(String name, boolean nullable, Predicate<Array<DataObject>> constraint);

	DataTypeBuilder addAnonymousProperty(String name, boolean reverse, boolean nullable, Predicate<DataObject> constraint);

	<T> DataTypeBuilder addArrayProperty(String name, Class<T> elementType);

	<T> DataTypeBuilder addArrayProperty(String name, Class<T> elementType, boolean nullable);

	<T> DataTypeBuilder addArrayProperty(String name, Class<T> elementType, boolean nullable, Predicate<Array<T>> constraint);

	<T> DataTypeBuilder addArrayProperty(String name, DataType elementType);

	DataTypeBuilder addArrayProperty(String name, DataType elementType, boolean nullable);

	DataTypeBuilder addArrayProperty(String name, DataType elementType, boolean nullable, Predicate<Array<DataObject>> constraint);

	DataTypeBuilder addBooleanProperty(String name);

	DataTypeBuilder addBooleanProperty(String name, boolean nullable);

	DataTypeBuilder addBooleanProperty(String name, boolean nullable, Predicate<Boolean> constraint);

	DataTypeBuilder addByteProperty(String name);

	DataTypeBuilder addByteProperty(String name, boolean nullable);

	DataTypeBuilder addByteProperty(String name, boolean nullable, Predicate<Byte> constraint);

	DataTypeBuilder addDateTimeProperty(String name);

	DataTypeBuilder addDateTimeProperty(String name, boolean nullable);

	DataTypeBuilder addDateTimeProperty(String name, boolean nullable, Predicate<DateTime> constraint);

	DataTypeBuilder addDecimalProperty(String name);

	DataTypeBuilder addDecimalProperty(String name, boolean nullable);

	DataTypeBuilder addDecimalProperty(String name, boolean nullable, Predicate<BigDecimal> constraint);

	DataTypeBuilder addDOProperty(String name, DataType type);

	DataTypeBuilder addDOProperty(String name, DataType type, boolean reverse);

	DataTypeBuilder addDOProperty(String name, DataType type, boolean reverse, boolean nullable);

	DataTypeBuilder addDOProperty(String name, DataType type, boolean reverse, boolean nullable, Predicate<DataObject> constraint);

	DataTypeBuilder addDoubleProperty(String name);

	DataTypeBuilder addDoubleProperty(String name, boolean nullable);

	DataTypeBuilder addDoubleProperty(String name, boolean nullable, Predicate<Double> constraint);

	DataTypeBuilder addDurationProperty(String name);

	DataTypeBuilder addDurationProperty(String name, boolean nullable);

	DataTypeBuilder addDurationProperty(String name, boolean nullable, Predicate<Duration> constraint);

	DataTypeBuilder addFloatProperty(String name);

	DataTypeBuilder addFloatProperty(String name, boolean nullable);

	DataTypeBuilder addFloatProperty(String name, boolean nullable, Predicate<Float> constraint);

	DataTypeBuilder addIntegerProperty(String name);

	DataTypeBuilder addIntegerProperty(String name, boolean nullable);

	DataTypeBuilder addIntegerProperty(String name, boolean nullable, Predicate<Integer> constraint);

	DataTypeBuilder addLongProperty(String name);

	DataTypeBuilder addLongProperty(String name, boolean nullable);

	DataTypeBuilder addLongProperty(String name, boolean nullable, Predicate<Long> constraint);

	DataTypeBuilder addPeriodProperty(String name);

	DataTypeBuilder addPeriodProperty(String name, boolean nullable);

	DataTypeBuilder addPeriodProperty(String name, boolean nullable, Predicate<Period> constraint);

	<T> DataTypeBuilder addProperty(String name, boolean nullable, Class<T> type, Predicate<T> constraint);

	<T> DataTypeBuilder addProperty(UndefinedProperty<T> property);

	DataTypeBuilder addShortProperty(String name);

	DataTypeBuilder addShortProperty(String name, boolean nullable);

	DataTypeBuilder addShortProperty(String name, boolean nullable, Predicate<Short> constraint);

	DataTypeBuilder addStringProperty(String name);

	DataTypeBuilder addStringProperty(String name, boolean nullable);

	DataTypeBuilder addStringProperty(String name, boolean nullable, Predicate<String> constraint);

	DataTypeBuilder allowExtensions();

	DataType getType();

	DataTypeBuilder setParentType(DataType parentType);

	String getName();

}
