package net.sf.sido.factory.json;

import net.sf.sido.DataType;
import net.sf.sido.DataTypeBuilder;

import org.codehaus.jackson.JsonNode;

public class JSONTypeDefinition {

	private final String name;
	private final DataTypeBuilder builder;
	private final JsonNode node;

	private JSONResolutionStatus status;
	private DataType type;

	public JSONTypeDefinition(String name, DataTypeBuilder builder, JsonNode node) {
		this.name = name;
		this.builder = builder;
		this.node = node;
		this.status = JSONResolutionStatus.PENDING;
	}

	public void complete() {
		if (this.status == JSONResolutionStatus.COMPLETE || this.status == JSONResolutionStatus.PENDING) {
			throw new IllegalStateException(String.format("Status %s for definition %s cannot be completed.", this.status, this.name));
		} else if (this.status == JSONResolutionStatus.ONGOING) {
			this.type = this.builder.getType();
			this.status = JSONResolutionStatus.COMPLETE;
			this.type.close();
		} else {
			throw new IllegalStateException(String.format("Status %s is unknown.", this.status));
		}
	}

	public DataTypeBuilder getBuilder() {
		return this.builder;
	}

	public String getName() {
		return this.name;
	}

	public JsonNode getNode() {
		return this.node;
	}

	public JSONResolutionStatus getStatus() {
		return this.status;
	}

	public DataType getType() {
		return this.type;
	}

	public void setParentType(DataType parentType) {
		this.builder.setParentType(parentType);
	}

	public void start() {
		if (this.status == JSONResolutionStatus.ONGOING || this.status == JSONResolutionStatus.COMPLETE) {
			throw new IllegalStateException(String.format("Status %s for definition %s cannot be started.", this.status, this.name));
		} else if (this.status == JSONResolutionStatus.PENDING) {
			this.status = JSONResolutionStatus.ONGOING;
		} else {
			throw new IllegalStateException(String.format("Status %s is unknown.", this.status));
		}
	}

}
