package net.sf.sido.factory.json;

import org.apache.commons.lang.StringUtils;
import org.codehaus.jackson.JsonNode;

public class JSONUtils {

	public static boolean getBoolean(JsonNode jNode, String name,
			boolean required, boolean defaultValue) {
		String value = getString(jNode, name, required, null);
		if (value == null) {
			return defaultValue;
		} else {
			return Boolean.valueOf(value);
		}
	}

	public static int getInteger(JsonNode jNode, String name, boolean required,
			int defaultValue) {
		String value = getString(jNode, name, required, null);
		if (value == null) {
			return defaultValue;
		} else {
			return Integer.parseInt(value, 10);
		}
	}

	public static long getLong(JsonNode jNode, String name, boolean required,
			long defaultValue) {
		String value = getString(jNode, name, required, null);
		if (value == null) {
			return defaultValue;
		} else {
			return Long.parseLong(value, 10);
		}
	}

	public static String getString(JsonNode jNode, String name) {
		JsonNode jField = jNode.get(name);
		if (jField != null) {
			return jField.getValueAsText();
		} else {
			return null;
		}
	}

	public static String getString(JsonNode jNode, String name,
			boolean required, String defaultValue) {
		String value = getString(jNode, name);
		if (StringUtils.isBlank(value)) {
			if (required) {
				throw new DOJsonFieldNotFoundException(name);
			} else {
				return defaultValue;
			}
		} else {
			return value;
		}
	}
}
