package net.sf.sido.factory.sido;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import net.sf.sido.DataFactory;
import net.sf.sido.DataSchema;
import net.sf.sido.factory.DataFactoryReader;
import net.sf.sido.factory.sido.antlr.SidoLexer;
import net.sf.sido.factory.sido.antlr.SidoParser;
import net.sf.sido.factory.sido.antlr.SidoReporter;
import net.sf.sido.factory.sido.antlr.StdErrorReporter;

import org.antlr.runtime.ANTLRInputStream;
import org.antlr.runtime.CommonTokenStream;
import org.antlr.runtime.RecognitionException;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.Validate;

public class SiDOLDataFactoryReader implements DataFactoryReader {

	@Override
	public DataSchema read(URL url, DataFactory factory) throws IOException {
		// Checks
		Validate.notNull(url, "The schema URL is required");
		Validate.notNull(factory, "The schema factory is required");
		// Gets the URL as stream
		InputStream in = url.openStream();
		if (in == null) {
			throw new IOException(String.format("Cannot open resource at %s", url));
		} else {
			try {
				// ANTLR stream
				ANTLRInputStream antltStream = new ANTLRInputStream(in, "UTF-8");
				// Error reporter
				StdErrorReporter errorReporter = new StdErrorReporter();
				// Base reporter
				SidoReporter sido = new SidoReporter(factory);
				// Lexer
				SidoLexer lex = new SidoLexer(antltStream);
				// Prepares the parsing
				CommonTokenStream tokens = new CommonTokenStream(lex);
				// Parser
				SidoParser parser = new SidoParser(tokens);
				parser.setErrorReporter(errorReporter);
				parser.setSidoReporter(sido);
				// Parsing
				try {
					parser.schema();
				} catch (RecognitionException e) {
					throw new IOException("Cannot parse " + url, e);
				}
				// Checks the errors
				if (errorReporter.hasErrors()) {
					// Gets all errors in one block
					String errors = StringUtils.join(errorReporter.getErrors(), "\n");
					// OK
					throw new IOException(String.format("SiDOL parsing error on %s%n%s", url, errors));
				}
				// OK
				return sido.getSchema();
			} finally {
				in.close();
			}
		}
	}

}
