package net.sf.sido.factory.sido.antlr;

import net.sf.sido.DOTypeNotFoundException;
import net.sf.sido.DataFactory;
import net.sf.sido.DataObject;
import net.sf.sido.DataSchema;
import net.sf.sido.DataType;
import net.sf.sido.DataTypeBuilder;
import net.sf.sido.array.Array;

import org.apache.commons.lang.StringUtils;

import com.google.common.base.Predicate;

public class TypePropertyContainer extends AbstractPropertyContainer {

	private final DataTypeBuilder builder;

	public TypePropertyContainer(DataSchema schema, String name) {
		super(schema);
		builder = schema.createDataTypeBuilder(name);
		// Registers the type in the schema
		builder.getType();
	}

	public void setAbstract() {
		builder.abstractType();
	}

	public void setExtensible() {
		builder.allowExtensions();
	}

	public void setParentTypeName(String parentTypeName) {
		// Looks for the type
		DataType parentType = getType(parentTypeName);
		if (parentType == null) {
			throw new DOTypeNotFoundException(parentTypeName);
		}
		// Sets the parent type
		builder.setParentType(parentType);
	}

	@Override
	protected void resolveProperty(Property property) {
		DataFactory factory = schema.getFactory();
		// Property declaration
		String name = property.getName();
		String typeName = property.getTypeName();
		boolean array = property.isArray();
		boolean nullable = property.isNullable();
		boolean reverse = property.isReverse();
		// Default type
		if (StringUtils.isBlank(typeName)) {
			typeName = "string";
		}
		// Constraint
		Predicate<?> constraint = factory.getConstraintFactory().createConstraint(property.getConstraintDefinitions());
		// Qualified type?
		DataType doType = getType(typeName);
		// Anonymous
		if (property.isAnonymous()) {
			// Array
			if (array) {
				@SuppressWarnings("unchecked")
				Predicate<Array<DataObject>> thisConstraint = (Predicate<Array<DataObject>>) constraint;
				builder.addAnonymousArrayProperty(name, nullable, thisConstraint);
			} else {
				@SuppressWarnings("unchecked")
				Predicate<DataObject> thisConstraint = (Predicate<DataObject>) constraint;
				builder.addAnonymousProperty(name, reverse, nullable, thisConstraint);
			}
		} else {
			// Simple type
			boolean simpleType = (doType == null);
			if (simpleType) {
				Class<?> typeClass = factory.getSimpleType(typeName);
				// Array
				if (array) {
					// Simple array
					addArrayProperty(name, nullable, typeClass, constraint);
				}
				// Single
				else {
					// Simple
					addSimpleProperty(name, nullable, typeClass, constraint);
				}
			}
			// DO type
			else {
				// Array
				if (array) {
					@SuppressWarnings("unchecked")
					Predicate<Array<DataObject>> thisConstraint = (Predicate<Array<DataObject>>) constraint;
					builder.addArrayProperty(name, doType, nullable, thisConstraint);
				}
				// Single
				else {
					@SuppressWarnings("unchecked")
					Predicate<DataObject> thisConstraint = (Predicate<DataObject>) constraint;
					builder.addDOProperty(name, doType, reverse, nullable, thisConstraint);
				}
			}
		}
	}

	protected <T> DataTypeBuilder addSimpleProperty(String name, boolean nullable, Class<T> typeClass,
			Predicate<?> constraint) {
		@SuppressWarnings("unchecked")
		Predicate<T> thisConstraint = (Predicate<T>) constraint;
		return builder.addProperty(name, nullable, typeClass, thisConstraint);
	}

	protected <T> void addArrayProperty(String name, boolean nullable, Class<T> typeClass, Predicate<?> constraint) {
		@SuppressWarnings("unchecked")
		Predicate<Array<T>> thisConstraint = (Predicate<Array<T>>) constraint;
		builder.addArrayProperty(name, typeClass, nullable, thisConstraint);
	}

}
